
import isystem.connect as ic
import time

cmgr = ic.ConnectionMgr()
cmgr.connectMRU('')
ideCtrl = ic.CIDEController(cmgr)
dataCtrl = ic.CDataController(cmgr)

memoryAreas = dataCtrl.getSystemMemoryAreas()
maPhysical = memoryAreas.getMemAreaDataPhysical()
amRealTime = ic.IConnectDebug.fRealTime

value = 0

valueType = ic.SType()
valueType.m_byType = ic.SType.tUnsigned  
valueType.m_byBitSize = 32

SRC_BMR2        = 0xFBFC4004
SRC_BMR2_BOOTMODE_MASK          = 0x06000000
SRC_BMR2_BOOTMODE_DEVELOPMENT   = 0x04000000
SRC_BMR2_CSE_SEC_BOOT_MODE_MASK = 0x00000003
SRC_BMR2_CSE_SEC_BOOT_MODE_NSC  = 0x00000003
SRC_SEC_BOOT    = 0xFBFC400C
SRC_SEC_BOOT_BOOTROM_PROT_MASK  = 0x00000001
SRC_SEC_BOOT_BOOTROM_PROT       = 0x00000001
MC_RGM_DES      = 0xFFFA8000
MC_RGM_FES      = 0xFFFA8300
MC_RGM_FRET     = 0xFFFA8604
MC_RGM_DRET     = 0xFFFA8608
MC_ME_MCTL      = 0xFFFB8004
OCOTP_SHADOW_16 = 0xFFC30240

# ------------------------------
# helper functions 
#
def Read32(address):
    return dataCtrl.readValue(amRealTime, maPhysical, address, valueType)

def Write32(address,value):
    dataCtrl.writeValue(amRealTime, maPhysical, address,  ic.CValueType(valueType, value)) 
    
HWBP_IAC7_Address = '';
HWBP_IAC8_Address = '';
HWBP_IAC7_Enable  = 'FALSE'
HWBP_IAC8_Enable  = 'FALSE'
HWBP_IAC_Combination  =''
HWBP_Enabled      = 'FALSE'

def ReadBreakpointRegisters():
  global HWBP_IAC7_Address
  global HWBP_IAC8_Address
  global HWBP_IAC7_Enable
  global HWBP_IAC8_Enable
  global HWBP_IAC_Combination
  global HWBP_Enabled

  HWBP_IAC7_Address    =ideCtrl.getOptionStr('/iOPEN/Breakpoints.PPC55xx.BP.e200[0].IAC_Address[6]')
  HWBP_IAC8_Address    =ideCtrl.getOptionStr('/iOPEN/Breakpoints.PPC55xx.BP.e200[0].IAC_Address[7]')
  HWBP_IAC7_Enable     =ideCtrl.getOptionStr('/iOPEN/Breakpoints.PPC55xx.BP.HW.e200[0].IAC_Enable[6]')
  HWBP_IAC8_Enable     =ideCtrl.getOptionStr('/iOPEN/Breakpoints.PPC55xx.BP.HW.e200[0].IAC_Enable[7]')
  HWBP_IAC_Combination =ideCtrl.getOptionStr('/iOPEN/Breakpoints.PPC55xx.BP.HW.e200[0].IAC_Combination[3]')
  HWBP_Enabled         =ideCtrl.getOptionStr('/iOPEN/Breakpoints.PPC55xx.BP.HW.e200[0].Enabled')
            

def RestoreBreakpointRegisters():
  ideCtrl.setOption('/iOPEN/Breakpoints.PPC55xx.BP.e200[0].IAC_Address[6]',       HWBP_IAC7_Address)  
  ideCtrl.setOption('/iOPEN/Breakpoints.PPC55xx.BP.e200[0].IAC_Address[7]',       HWBP_IAC8_Address)
  ideCtrl.setOption('/iOPEN/Breakpoints.PPC55xx.BP.HW.e200[0].IAC_Enable[6]',     HWBP_IAC7_Enable)
  ideCtrl.setOption('/iOPEN/Breakpoints.PPC55xx.BP.HW.e200[0].IAC_Enable[7]',     HWBP_IAC8_Enable)  
  ideCtrl.setOption('/iOPEN/Breakpoints.PPC55xx.BP.HW.e200[0].IAC_Combination[3]',HWBP_IAC_Combination)
  ideCtrl.setOption('/iOPEN/Breakpoints.PPC55xx.BP.HW.e200[0].Enabled',           HWBP_Enabled)
  ideCtrl.serviceCall("/IOPEN/Core.Debug.ApplyInternalBPs", "")

def SetBreakpointToRAMArea():
  #Set global HWBP enable, enable IAC7/8, set start address to IAC7, end address to IAC8, combination 'inside'
  ideCtrl.setOption('/iOPEN/Breakpoints.PPC55xx.BP.HW.e200[0].Enabled',            'TRUE')
  ideCtrl.setOption('/iOPEN/Breakpoints.PPC55xx.BP.HW.e200[0].IAC_Enable[6]',      'TRUE')
  ideCtrl.setOption('/iOPEN/Breakpoints.PPC55xx.BP.HW.e200[0].IAC_Enable[7]',      'TRUE')
  ideCtrl.setOption('/iOPEN/Breakpoints.PPC55xx.BP.e200[0].IAC_Address[6]',        '0x01000000')
  ideCtrl.setOption('/iOPEN/Breakpoints.PPC55xx.BP.e200[0].IAC_Address[7]',        '0x0127FFFF')
  ideCtrl.setOption('/iOPEN/Breakpoints.PPC55xx.BP.HW.e200[0].IAC_Combination[3]', 'Inside')
  ideCtrl.serviceCall("/IOPEN/Core.Debug.ApplyInternalBPs", "")
  
def InvalidateContenxt():
  ideCtrl.serviceCall("/IOPEN/HW.SoC.StateControl", "Operation: InvalidateContextFromTarget, Scope: Core")
#
# ------------------------------

#
# ******************************
# start of cript
# 

print("#                                                                                        ");
print("#  This script enables non-secure QSPI boot on S32R294.                                  ");
print("#  It can handle device samples which are fused for secure boot (factory default for S2) ");
print("#                                                                                        ");

LatchRESET= ideCtrl.getOptionStr('/iOPEN/Emulation.Debugging.LatchRESET')
if 'TRUE'==LatchRESET:
    print("Aborting operation.");
    print("Disable'Latch target RESET' option in winIDEA");
else:    
    # Check the SECURE_BOOT_MODE fuse. We only perform override if we have to.
    print("Reading SRC registers");
    
    value = Read32(SRC_SEC_BOOT)
    print("-SRC_SEC_BOOT: {}".format(hex(value.getInt()))) 
    
    value = Read32(SRC_BMR2)
    print("-SRV_BMR2: {}".format(hex(value.getInt()))) 
    
    if SRC_BMR2_BOOTMODE_DEVELOPMENT == (value.getInt() & SRC_BMR2_BOOTMODE_MASK):
    
      if SRC_BMR2_CSE_SEC_BOOT_MODE_NSC == (value.getInt() & SRC_BMR2_CSE_SEC_BOOT_MODE_MASK):
        print("Device is already fused for non-secure boot. Aborting override.") 
        
        value = Read32(SRC_SEC_BOOT)
        print("Checking Boot ROM execution status");
    
        if SRC_SEC_BOOT_BOOTROM_PROT == (value.getInt() & SRC_SEC_BOOT_BOOTROM_PROT_MASK):  
          print(" Boot ROM was already executed.")
        else:
          print(" Boot ROM was not executed. Perform a target power cycle.")
            
      else:  
        print("Starting non-secure boot from QSPI Flash") 
        print(" -reset RGM reset flags") 
        Write32(MC_RGM_DES, 0xFFFFFFFF)
        Write32(MC_RGM_FES, 0xFFFFFFFF)
        
        print(" -disable reset escalation")
        Write32(MC_RGM_FRET,0x00000000) 
        Write32(MC_RGM_DRET,0x00000000) 
        
        print(" -write OCOTP shadow register [16] to override secure boot (for non-secure boot after reset)")
        Write32(OCOTP_SHADOW_16, 0x000000C0)
        
        print(" -read breakpoint registers")
        ReadBreakpointRegisters()
        
        print(" -set range breakpoint from 0x01000000 to 0x0127FFFF")
        SetBreakpointToRAMArea()
        
        print(" -execute mode change")
        Write32(MC_ME_MCTL, 0x00005AF0)
        Write32(MC_ME_MCTL, 0x0000A50F)
    
        print(" -wait 500ms to be finished")
        time.sleep(0.5)
        
        print(" -read SRC registers");
        value = Read32(SRC_BMR2)
        print("  -SRC_BMR2: {}".format(hex(value.getInt()))) 
        value = Read32(SRC_SEC_BOOT)
        print("  -SRC_SEC_BOOT: {}".format(hex(value.getInt()))) 
        if SRC_SEC_BOOT_BOOTROM_PROT == (value.getInt() & SRC_SEC_BOOT_BOOTROM_PROT_MASK):  
            print("Non-secure QSPI flash boot complete")
        
        print(" -restore breakpoints")    
        RestoreBreakpointRegisters();
        
        print(" -invalidate context")    
        InvalidateContenxt()
      
    else:
      print("Wrong BOOTMODE0/ BOOTMODE1 detected") 

ideCtrl.refreshUI()
input('Press ENTER to exit')

#
# ******************************